@@ -1,8 +1,8 @@
 # Module                       : mh_universal_functions.py
 # Related program              : None
 # Author                       : MH
-# Date                         : 2020.06.28
-# Version                      : 2020.06.29.2
+# Date                         : see _version.py
+# Version                      : see _version.py
 # Python Version, Mac OS X     : 3.7.7
 # Python Version, Raspberry Pi : 3.7.7
 
@@ -12,17 +12,22 @@
 
 
 # Required packages in PyCharm and Raspberry Pi:
-# Name          Type              Mac-Version  Raspi-Version  Dependencies  VersionCheckCommand
+# Name             Type              Mac-Version  Raspi-Version  Dependencies  VersionCheckCommand
 # ----------------------------------------------------------------------------------------------------------------------
-# configparser  standard-library  3.7.7        3.7.7          -             -
-# datetime      standard-library  3.7.7        3.7.7          -             -
-# logging       standard-library  3.7.7        3.7.7          -             -
-# os            standard-library  3.7.7        3.7.7          -             -
-# pymysql       external          0.9.3        0.9.3          -             print(pymysql.__version__)
-# subprocess    standard-library  3.7.7        3.7.7          -             -
-# sys           standard-library  3.7.7        3.7.7          -             print(sys.version)
-# threading     standard-library  3.7.7        3.7.7          -             -
+# pybluez          external            0.23         0.23         -             print(bluetooth.__version__)
+# configparser     standard-library   3.7.7        3.7.7         -             -
+# datetime         standard-library   3.7.7        3.7.7         -             -
+# fritzconnection  external           1.3.4        0.23          -             print(bluetooth.__version__)
+# logging          standard-library   3.7.7        3.7.7         -             -
+# os               standard-library   3.7.7        3.7.7         -             -
+# pymysql          external          0.10.1       0.10.1         -             print(pymysql.__version__)
+# subprocess       standard-library   3.7.7        3.7.7         -             -
+# sys              standard-library   3.7.7        3.7.7         -             print(sys.version)
+# threading        standard-library   3.7.7        3.7.7         -             -
 
+# Module "bluetooth": The PyBluez module allows Python code to access the host machine's Bluetooth resources.
+# (https://github.com/pybluez/pybluez)
+import bluetooth
 
 # Module "configparser": Configuration file parser
 # (https://docs.python.org/3.7/library/configparser.html#module-configparser)
@@ -32,6 +37,10 @@ from configparser import ConfigParser
 # (https://docs.python.org/3.7/library/datetime.html#module-datetime)
 from datetime import datetime
 
+# Module "fritzconnection": Basic date and time types
+# (https://pypi.org/project/fritzconnection/)
+from fritzconnection import FritzConnection
+
 # Module "logging": Logging facility for Python
 # (https://docs.python.org/3.7/library/logging.html#module-logging)
 import logging
@@ -44,11 +53,9 @@ from os import path
 # (https://pypi.org/project/PyMySQL)
 # Using "pymysql" instead of MySQLdb which is limited to python2.7
 from pymysql import connect
-from pymysql import Error
 
 # Module "subprocess": Subprocess management
 # (https://docs.python.org/3.7/library/subprocess.html#module-subprocess)
-from subprocess import CalledProcessError
 from subprocess import check_call
 
 # Import general modules
@@ -119,7 +126,7 @@ def database_update_universal(mysql_const, row, table, value, id1, id1_value, id
         db_connection.commit()
         db_connection.close()
         return True
-    except Error as error_detail1:
+    except Exception as error_detail1:
         module_logger.error("Cannot connect do database, error_detail1 = '" + str(error_detail1) + "'")
         return False
 
@@ -181,7 +188,7 @@ def database_read(mysql_const, row, table, id1, id1_value, id2, id2_value):
         value = cursor.fetchone()[0]
         db_connection.close()
         return value
-    except Error as error_detail1:
+    except Exception as error_detail1:
         module_logger.error("Cannot connect to MySQL database, error_detail2 = '" + str(error_detail1) + "'")
         return "error"
 
@@ -259,7 +266,7 @@ def send_telegram_message(telegram_active, telegram_program, telegram_recipient_
 # ToDO: think about adding _telegram_message_address to above module_logger message...
 
                 return True
-            except CalledProcessError as error_detail3:
+            except Exception as error_detail3:
                 module_logger.error("Telegram message_text send: failed, error_detail3 = '" + str(error_detail3) + "'")
                 return False
         else:
@@ -277,5 +284,69 @@ def send_telegram_message(telegram_active, telegram_program, telegram_recipient_
     telegram_message_thread.start()
 
 
+def get_mobile_device_presence(_wifi_devices_dict, _router_ip_address, _router_password,
+                               _bluetooth_devices_dict,
+                               _scan_for_device_type):
+    """
+    Evaluates whether a bluetooth of WiFi device can be detected.
+    :param: _wifi_devices_dict: dict, contains the wifi devices name and Mac address
+    :param: _router_ip_address: string, contains the routers ip address
+    :param: _router_password: string, contains the routers password
+    :param: _bluetooth_devices_dict: dict, contains the bluetooth devices name and Mac address
+    :param: _scan_for_device_type: string, contains the scan type "bluetooth" or "WiFi" or "both"
+    :returns: bool, device in dictionaries _wifi_devices_dict or  _bluetooth_devices_dict are present
+    """
+
+    def scan_bluetooth():
+        if platform == "linux" or platform == "linux2":
+            for key in iter(_bluetooth_devices_dict.keys()):
+                device_present = bluetooth.lookup_name(str(_bluetooth_devices_dict[key]), timeout=5)
+                if device_present is not None:
+                    return key, True
+            return None, False
+        else:
+            return None, False
+
+    def scan_wifi():
+        fritz_connection = FritzConnection(address=_router_ip_address, password=_router_password)
+        host_numbers = fritz_connection.call_action('Hosts', 'GetHostNumberOfEntries')['NewHostNumberOfEntries']
+        index = 0
+        while index < host_numbers:
+            host = fritz_connection.call_action('Hosts', 'GetGenericHostEntry', NewIndex=index)
+            for key in iter(_wifi_devices_dict.keys()):
+                device_present = _wifi_devices_dict[key]
+                if (host['NewMACAddress'] == device_present) and (host['NewActive']):
+                    return key, True
+            index = index + 1
+        return None, False
+
+    def mobile_device_presence():
+        if _scan_for_device_type == "bluetooth":
+            return scan_bluetooth()
+        elif _scan_for_device_type == "wifi":
+            return scan_wifi()
+        elif _scan_for_device_type == "both":
+            bluetooth_key, bluetooth_bool = scan_bluetooth()
+            wifi_key, wifi_bool = scan_wifi()
+
+            if bluetooth_key is None and wifi_key is None:
+                module_logger.info("def 'mobile_device_presence' returns: None and False")
+                return None, False
+            elif bluetooth_key is not None and wifi_key is not None:
+                module_logger.info("def 'mobile_device_presence' returns: '" + bluetooth_key + "' and '" +
+                                   str(bluetooth_bool) + "'")
+                return bluetooth_key, bluetooth_bool
+            elif bluetooth_key is None and wifi_key is not None:
+                module_logger.info("def 'mobile_device_presence' returns: '" + wifi_key + "' and '" +
+                                   str(wifi_bool) + "'")
+                return wifi_key, wifi_bool
+            elif bluetooth_key is not None and wifi_key is None:
+                module_logger.info("def 'mobile_device_presence' returns: '" + bluetooth_key + "' and '" +
+                                   str(bluetooth_bool) + "'")
+                return bluetooth_key, bluetooth_bool
+
+    return mobile_device_presence
+
+
 def test():
     print("Hello world of PyPI")
